//<c>   Copyright 1997-1999 by Gerry J. Danen; all rights reserved.

//<t>   Nuke Old Nodelists

#include"gd_tools.h"
#include<stdarg.h>


#define pgVERS_ "V1.20"
#define pgTITL_ "Nuke Old Nodelists"
#define pgCOPR_ "1997"


UCHAR   iniSec[]="Nuke-NL", iniFile[FS_LEN], fileList[3001], fileName[FS_LEN],
        nPath[FS_LEN], logFile[FS_LEN], work[1001], work1[1001], work2[1001] ;
char    origdir[FS_LEN] ;
BOOL    ynVerbose = FALSE, ynTesting = FALSE,
        ynLogStd = FALSE, ynLogVerbose = FALSE, ynLogDebug = FALSE ;
FILE    *lF ;


void    program_help( void )
{
    logon(1);
    printf( "\n"
            "%s is a utility program for use with Fido-style networks.\n"
            "It's purpose is to delete all old nodelist files, using parameters\n"
            "from %s.INI.\n"
            "The ONLY files processed, are those with a 3-digit extention!\n\n"
            "Usage:\n"
            "    %s\n\n",
            get_global_program_name(),
            get_global_program_name(), get_global_program_name() );
    exit( 0 );
}


void    wrlog( UCHAR typ, STRING msg, ... )
{
    UCHAR       mbuf[161];
    va_list     arg_ptr ;
    char        mytype = UP_UCHAR(typ);         // Std, Verbose, or Debug
    char        nutype ;                        // :    +           $

    if ( ! ynLogStd )                           // standard logging is always set if ANYTHING is being logged
        return ;
    if ( mytype == 'D' && (! ynLogDebug) )      // skip debug info
        return ;
    if ( mytype == 'V' && (! ynLogVerbose) )    // skip verbose info
        return ;

    va_start( arg_ptr, msg );
    vsprintf( mbuf, msg, arg_ptr );
    va_end( arg_ptr );

    switch ( mytype )
    {
        case 'D' :
            nutype = '$' ;
            break ;
        case 'V' :
            nutype = '+' ;
            break ;
        default :
            nutype = ':' ;
            break ;
    }

    fprintf( lF, "%c %s %s %s  %s\n",
                 nutype, TodaysCDate(), TodaysTime(),
                 get_global_program_name(), mbuf );
}


void    process( STRING fn )
{
    struct _find_t nl ;
    unsigned    done, attrib = REGULAR_FILES ;  // all except directories, and volid
    char        sToFind[FS_LEN] ;               // file spec to search for
    char        fSpec[FS_LEN] ;                 // file spec to keep
    int         x, day, month, year, hour, minute, secs ;
    INT32       ymdC=-1, ymdP=-1, hmsC=-1, hmsP=-1 ;

    if ( ynVerbose )
    {
        printf( "Processing %s:\n", fn );
        wrlog( 'D', "Processing %s", fn );
    }

    sprintf( sToFind, "%s.*", fn );

    done = _dos_findfirst( sToFind, attrib, &nl );  // phase 1 - see what we have

    if ( done )                                 // no files
    {
        if ( ynVerbose )
        {
            printf( "           ---no files match %s.nnn---\n", fn );
            wrlog( 'V', "No files match %s.nnn", fn );
        }
        return ;
    }

    while ( ! done )
    {
        _strlwr( nl.name );
        strcpy( work, nl.name );

        x = strlen(fn) + 1 ;
        if ( isdigit(work[x])   &&
             isdigit(work[x+1]) &&
             isdigit(work[x+2]) )
        {
            Fstamp2date( nl.wr_date, &year, &month, &day );
            Fstamp2time( nl.wr_time, &hour, &minute, &secs );

            ymdC = dt_ymd( day, month, year );
            hmsC = hour*10000L + minute*100L + secs ;

            if ( (ymdC > ymdP) ||
                (ymdC == ymdP && hmsC > hmsP) )
            {
                ymdP = ymdC ;
                hmsP = hmsC ;
                strcpy( fSpec, nl.name );
            }
        }
        done = _dos_findnext( &nl );           // find next file
    }

    done = _dos_findfirst( sToFind, attrib, &nl );  // phase 2 - do the actual work
    while ( ! done )
    {
        _strlwr( nl.name );
        strcpy( work, nl.name );

        x = strlen(fn) + 1 ;
        if ( isdigit(work[x])   &&
             isdigit(work[x+1]) &&
             isdigit(work[x+2]) )
        {
            Fstamp2date( nl.wr_date, &year, &month, &day );
            Fstamp2time( nl.wr_time, &hour, &minute, &secs );
            ymdC = dt_ymd( day, month, year );
            hmsC = hour*10000L + minute*100L + secs ;
            FormatDate( ymdC, work1, 9 );
            FormatTime( hmsC, work2, 1, 2 );

            if ( st_eq(work, fSpec) )
            {
                if ( ynVerbose )
                {
                    printf( "%10s - Saving %-13s %s  %s\n", "", work, work1, work2 );
                    wrlog( 'V', "Keeping %s %s %s", work, work1, work2 );
                }
            }
            else
            {
                strupr( work );
                if ( ynVerbose )
                    printf( "%10s - NUKING %-13s %s  %s\n", "", work, work1, work2 );
                else
                    printf( "Deleting %s\n", work );

                if ( ynLogVerbose )
                    wrlog( 'V', "Deleting %s %s %s", work, work1, work2 );
                else
                    wrlog( 's', "Deleting %s", work );

                if ( ! ynTesting )
                    fl_delete( work );
            }
        }
        done = _dos_findnext( &nl );           // find next file
    }
}


void    main_exit( void )
{
    unsigned    numofdrives ;

    _fcloseall();
    _dos_setdrive( origdir[0] - 64, &numofdrives );
    _chdir( origdir );                          // back to original
}


void    main( int argc, char *argv[] )
{
    int         xcnt = 0 ;

    tt_init( SHAREWARE_VERSION, 0, pgVERS_, pgTITL_, pgCOPR_ );
    logon(0);

    _getcwd( origdir, sizeof(origdir) );        // get current directory & drive
    atexit( main_exit );

    if ( st_eq(argv[1],"/help") || st_eq(argv[1],"/h") || st_eq(argv[1],"/?") )
        program_help();

    if ( fl_datafilename( iniFile, argv[0], "ini" ) == ERROR )
    {
        logon(1);
        printf( "Can't find %s.\a\n", iniFile );
        exit( 8 );
    }

    fl_parse_ini( iniFile, iniSec, "logFile", logFile, &xcnt );
    strupr( logFile );
    if ( strlen(logFile) )
    {
        if ( ( lF = fopen( logFile, "a" ) ) != NULL )
        {
            ynLogStd = TRUE ;
            fl_parse_ini( iniFile, iniSec, "logDebug", work, &xcnt );
            strupr( work );
            if ( work[0]=='Y' || work[0]=='T' )
                ynLogDebug = TRUE ;
            fl_parse_ini( iniFile, iniSec, "logVerbose", work, &xcnt );
            strupr( work );
            if ( work[0]=='Y' || work[0]=='T' )
                ynLogVerbose = TRUE ;
            if ( ynLogDebug )
                ynLogVerbose = TRUE ;
        }
        else
        {
            logon(1);
            printf( "\nERROR:\a CANNOT LOG ACTIONS TO %s\n\n", logFile );
        }
    }

    fl_parse_ini( iniFile, iniSec, "verbose", work, &xcnt );
    strupr( work );
    if ( work[0]=='Y' || work[0]=='T' )
        ynVerbose = TRUE ;
    fl_parse_ini( iniFile, iniSec, "testing", work, &xcnt );
    strupr( work );
    if ( work[0]=='Y' || work[0]=='T' )
        ynTesting = TRUE ;

    if ( ! ynLogVerbose )
        wrlog( 's', "---[ %s  %s  %s ]---", get_global_program_name(), __DATE__, __TIME__ );

    wrlog( 'D', "Program = %s [compiled %s @ %s]", argv[0], __DATE__, __TIME__ );
    wrlog( 'D', "Home directory = %s", origdir );

    if ( ynVerbose )
    {
        logon(1);
        printf( "Using settings from %s\n", iniFile );
        if ( ynLogStd )
            printf( "Logging actions to %s\n", logFile );
        if ( ynLogVerbose )
            printf( "  (logging extra information)\n" );
        if ( ynLogDebug )
            printf( "  (logging debug information)\n" );
    }
    wrlog( 'V', "Settings from %s <---", iniFile );

    if ( ynTesting )
    {
        printf( "TEST MODE - NOT REALLY DELETING FILES\n" );
        wrlog( 'D', "Test mode only" );
    }

    fl_parse_ini( iniFile, iniSec, "process", fileList, &xcnt );
    strupr( fileList );

    fl_parse_ini( iniFile, iniSec, "NodePath", nPath, &xcnt );
    strupr( nPath );
    if ( strlen(nPath) )                        // go to the directory
    {
        if ( ynVerbose )
            printf( "Switching to %s\n", nPath );
        wrlog( 'D', "Switching to %s", nPath );
        dr_mkdir( nPath, YES );
    }

    while ( strlen(fileList) )
    {
        st_get_word( fileName, fileList );      // get file name from list
        st_remove_word( fileList );             // remove from list
        process( fileName );
    }

    exit( 0 );
}
